/**
 * Note: This module cannot import any modules because it is meant to be a self-contained function.
 * This is to allow external libraries to access the rendererFactory, toString it and recreate it.
 * For example:
 *  import { rendererFactory } from 'lwc';
 *  import sanitize from 'sanitizeLibrary';
 *
 *  const sandboxedRendererFactory = sanitize(rendererFactory.toString());
 *
 *   Type-Only imports are allowed.
 */
import type { RendererAPI } from '@lwc/engine-core';
export type SandboxableRendererAPI = Omit<RendererAPI, 'createCustomElement' | 'insertStylesheet' | 'isNativeShadowDefined' | 'isSyntheticShadowDefined'>;
export type RendererAPIType<Type> = Type extends RendererAPI ? RendererAPI : SandboxableRendererAPI;
/**
 * A factory function that produces a renderer.
 * Renderer encapsulates operations that are required to render an LWC component into the underlying
 * runtime environment. In the case of @lwc/enigne-dom, it is meant to be used in a DOM environment.
 * Example usage:
 * import { renderer, rendererFactory } from 'lwc';
 * const customRenderer = rendererFactory(renderer);
 *
 * @param baseRenderer Either null or the base renderer imported from 'lwc'.
 */
export declare function rendererFactory<T extends RendererAPI | null>(baseRenderer: T): RendererAPIType<T>;
