import { ComponentReference, File, SourceLocation } from './common-metadata-types';
import { ModuleReference } from './script-metadata-types';
/**
 * Root metadata object for a css file.
 */
interface CSSFile extends File {
    fileType: 'css';
    /**
     * The list of imported CSS modules via the @import CSS at-rule.
     */
    imports: ModuleReference[];
    /**
     * The list of referenced component modules in CSS rules.
     */
    componentReferences: ComponentReference[];
    /**
     * The list of CSS custom properties.
     */
    customProperties: CSSCustomProperty[];
}
declare function isCSSFile(file: File): file is CSSFile;
/**
 * CSS custom property detail.
 */
interface CSSCustomProperty {
    /**
     * The custom property name.
     */
    name: string;
    /**
     * The custom property location.
     */
    location: SourceLocation;
}
export { CSSFile, CSSCustomProperty, isCSSFile };
