import { InstrumentationObject } from '@lwc/errors';
import { CustomRendererConfig } from './shared/renderer-hooks';
export interface Config {
    /**
     * Specification to use to determine which nodes in the template require custom renderer hooks.
     */
    customRendererConfig?: CustomRendererConfig;
    /**
     * Enable computed member expression in the template. eg:
     *    <template>
     *        {list[0].name}
     *    </template>
     */
    experimentalComputedMemberExpression?: boolean;
    /**
     * Enable use of (a subset of) JavaScript expressions in place of template bindings.
     *
     *    <template>
     *        <input
     *            attr={complex ?? expressions()}
     *            onchange={({ target }) => componentMethod(target.value)}
     *        >
     *            Hey there {inAustralia ? 'mate' : 'friend'}
     *        </input>
     *    </template>
     */
    experimentalComplexExpressions?: boolean;
    /**
     * TODO [#3331]: remove usage of lwc:dynamic in 246
     *
     * Enable lwc:dynamic directive - Deprecated
     *
     * <x-foo lwc:dynamic={expr}>
     */
    experimentalDynamicDirective?: boolean;
    /**
     * When true, enables `lwc:is` directive.
     *
     * <lwc:component lwc:is={expr}>
     */
    enableDynamicComponents?: boolean;
    /**
     * When true, HTML comments in the template will be preserved.
     */
    preserveHtmlComments?: boolean;
    /**
     * When false, the template compiler will not generate optimized code for static content.
     */
    enableStaticContentOptimization?: boolean;
    /**
     * @deprecated Spread operator is now always enabled.
     */
    enableLwcSpread?: boolean;
    /**
     * Config to use to collect metrics and logs
     */
    instrumentation?: InstrumentationObject;
}
export type NormalizedConfig = Required<Omit<Config, 'customRendererConfig' | 'instrumentation'>> & Partial<Pick<Config, 'customRendererConfig' | 'instrumentation'>>;
export declare function normalizeConfig(config: Config): NormalizedConfig;
