import { IHTMLTagProvider } from './parser/htmlTags';
import { TextDocument, Position, CompletionList, Hover, Range, SymbolInformation, TextEdit, DocumentHighlight, DocumentLink, FoldingRange } from 'vscode-languageserver-types';
import { Scanner, HTMLDocument, CompletionConfiguration, ICompletionParticipant, HTMLFormatConfiguration, DocumentContext } from './htmlLanguageTypes';
import { Location } from 'vscode-languageserver-types';
export * from './htmlLanguageTypes';
export * from 'vscode-languageserver-types';
export interface LanguageService {
    addTagProvider(provider: IHTMLTagProvider): void;
    getTagProviders(): IHTMLTagProvider[];
    createScanner(input: string, initialOffset?: number): Scanner;
    parseHTMLDocument(document: TextDocument): HTMLDocument;
    findDocumentHighlights(document: TextDocument, position: Position, htmlDocument: HTMLDocument): DocumentHighlight[];
    doComplete(document: TextDocument, position: Position, htmlDocument: HTMLDocument, options?: CompletionConfiguration): CompletionList;
    setCompletionParticipants(registeredCompletionParticipants: ICompletionParticipant[]): void;
    doHover(document: TextDocument, position: Position, htmlDocument: HTMLDocument): Hover | null;
    format(document: TextDocument, range: Range | undefined, options: HTMLFormatConfiguration): TextEdit[];
    findDocumentLinks(document: TextDocument, documentContext: DocumentContext): DocumentLink[];
    findDocumentSymbols(document: TextDocument, htmlDocument: HTMLDocument): SymbolInformation[];
    doTagComplete(document: TextDocument, position: Position, htmlDocument: HTMLDocument): string | null;
    getFoldingRanges(document: TextDocument, context?: {
        rangeLimit?: number;
    }): FoldingRange[];
    findDefinition(document: TextDocument, position: Position, htmlDocument: HTMLDocument): Location | null;
    getPropertyBindingTemplateDeclaration(document: TextDocument, position: Position, htmlDocument: HTMLDocument): Location | null;
    getPropertyBindingValue(document: TextDocument, position: Position, htmlDocument: HTMLDocument): string | null;
    getAuraBindingTemplateDeclaration(document: TextDocument, position: Position, htmlDocument: HTMLDocument): Location | null;
    getAuraBindingValue(document: TextDocument, position: Position, htmlDocument: HTMLDocument): string | null;
}
export declare function getLanguageService(): LanguageService;
//# sourceMappingURL=htmlLanguageService.d.ts.map