"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PREFIX = '@salesforce/schema/';
const PREFIX_LENGTH = PREFIX.length;
/**
 * Module resolver for @salesforce/schema/* that provides access to force.com SObjects and field identifiers.
 *
 * The supported module identifiers are dependent on the sObject schema in the org:
 * 1. sObject schema in a namespace: @salesforce/schema/[namespace]__[objectApiName]
 * 2. sObject schema in default ('c') namespace: @salesforce/schema/[objectApiName]
 * 3. Field schema in a namespace: @salesforce/schema/[namespace]__[objectApiName].[namespace]__[fieldApiName]
 * 4. Field schema in default ('c') namespace: @salesforce/schema/[objectApiName].[fieldApiName]
 * 5. Spanning field schema in a namespace: @salesforce/schema/[namespace]__[objectApiName].[namespace]__[relationshipApiName].[namespace]__[fieldApiName]
 * 6. Spanning field schema in default ('c') namespace: @salesforce/schema/[objectApiName].[relationshipApiName].[fieldApiName]
 *
 * @param resource The module identifier.
 * @returns The value of the requested resource.
 */
function schemaResolver(resource) {
    if (!resource.startsWith(PREFIX) || resource.length <= PREFIX_LENGTH) {
        return undefined;
    }
    resource = resource.substring(PREFIX_LENGTH);
    const idx = resource.indexOf('.');
    if (idx === -1) {
        return { objectApiName: resource };
    }
    const objectApiName = resource.substring(0, idx);
    const fieldApiName = resource.substring(idx + 1);
    return { fieldApiName, objectApiName };
}
exports.default = schemaResolver;
//# sourceMappingURL=index.js.map